package com.level3.mediaportal.api.exampleCode;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

/**
 * This class contains functions designed to send GET and POST
 * requests to the Media Portal API.
 * */
public class APICommunicatorJava {

	private String keyId;
	private String keySecret;
	
	/**
	 * Initializer for APICommunicator Object.
	 * 
	 * @param keyId	The ID of the API Key you'll be using to validate your API calls 
	 * @param keyId	The Secret of the API Key you'll be using to validate your API calls 
	 * */
	APICommunicatorJava(String keyId, String keySecret){
		this.keyId = keyId;
		this.keySecret = keySecret;
	}
	
	/**
	 * Computes the encoded authentication signature for a Media Portal API call. 
	 * An HMAC is created from the parameters sent, encoded in base 64, and returned 
	 * as a String.
	 * 
	 * @param dateStr    	Date in "%a, %d %b %Y %H:%M:%S %Z" format.
	 * @param uri    		Full URI of the service you wish to call (Not Including Parameters).
	 * @param contentType 	The format for which POST body parameters will be sent from the client. Typically "application/xml" or "application/json" (send an empty string if this is for a GET request).
	 * @param method    	The API method type ("GET," "POST," etc.) that you will be calling.
	 * @param md5    		The MD5 Hashed value of your post body (send an empty string if this is for a GET request or post body is null).
	 * 
	 * @return A string encoded in base64 to be added to an API call for authentication. Null if an error occurs while creating the signature.
	 * */
	private String getAuthenticationSignature(String dateStr, String uri, String contentType, String method, String md5){
		try {
			/*         
	         * The authentication signature is built as shown
	         *    Mon, 01 Jan 2018 00:00:00 GMT\n               
	         *    /api/uri/{access group ID}/{scid}/{property}\n
	         *    application/json\n                            
	         *    GET\n                                         
	         *    MD5 Hash (if applicable)                      
			 * */
			contentType = contentType == null? "":contentType;
			md5 = md5 == null? "":md5;
			String data =  dateStr +"\n"+ 
						   uri +"\n"+ 
						   contentType +"\n"+ 
						   method +"\n"+ 
						   md5;
		
			System.out.println("*****API Signature Pre-Hash*****");
			System.out.println(data);
			System.out.println("********************************\n");
			
			//Create the HMAC Hash of the request data
			SecretKeySpec keySpec = new SecretKeySpec(this.keySecret.getBytes(), "HmacSHA1");
			Mac mac;
			mac = Mac.getInstance("HmacSHA1");
			mac.init(keySpec);
			
			//Encode the HMAC in Base 64
	        byte[] dataBytes = data.getBytes("UTF-8");
	        byte[] hmacBytes = mac.doFinal(dataBytes);	        
	        String authSignature = Base64.getEncoder().encodeToString(hmacBytes);
			
			return authSignature;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Makes a GET service call to the Media Portal API.
	 * This demo includes information on how requests are built and sent, and includes
	 * information on creating authentication signatures (required in order to prove the
	 * user has necessary permissions to access the results).
	 * 
	 * @param accept	The format for which data from the service call should be returned. Typically "application/xml" or "application/json."
	 * @param dateStr	Date in "D, d M Y H:i:s T" format.
	 * @param url		Full URL of the service you wish to call (EX. https://fakeApiURL.com/services/key?param1=test1&param2=test2,test3).
	 * 
	 * @return A String containing the message returned by the call. Null if the call fails.
	 * */
	public String sendTestAPICallGET(String accept, String dateStr, String url) {
		try {
			URL parsedUrl = new URL(url);
			HttpURLConnection con = (HttpURLConnection) parsedUrl.openConnection(); 
			
			//Create the authorization property string for the request header.
	        //Format: MPA {key_id}:{signature}
			String signatureStr = getAuthenticationSignature(dateStr, parsedUrl.getPath(), "", "GET", "");
			String authStr = "MPA " + this.keyId + ":" + signatureStr;
			
			//Set Connection Method to GET
			con.setRequestMethod("GET");

			//Set Header Properties.
	        //Because this is a get, content-length == 0 and the properties content-type and content-md5 are not set.
			con.setRequestProperty("Accept", accept);
			con.setRequestProperty("Authorization", authStr);
			con.setRequestProperty("Date", dateStr);
			con.setRequestProperty("Content-Length", "0");
			con.setRequestProperty("Content-MD5", "");
			con.setRequestProperty("Content-Type", "");
			
			System.out.println("\n****Header Properties****");
			System.out.println("Accept : " + authStr);
			System.out.println("Authorization : " + authStr);
			System.out.println("Date : " + dateStr);
			System.out.println("Content-Length : 0");
			System.out.println("Content-MD5 : ");
			System.out.println("Content-Type : ");
			System.out.println("*************************\n");
			
			BufferedReader in = new BufferedReader(
			        new InputStreamReader(con.getInputStream()));
			String inputLine;
			StringBuffer response = new StringBuffer();

			while ((inputLine = in.readLine()) != null) {
				response.append(inputLine);
			}
			in.close();

			return response.toString();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Makes a POST service call to the Media Portal API.
	 * This demo includes information on how requests are built and sent, and includes
	 * information on creating authentication signatures (required in order to prove the
	 * user has necessary permissions to access the results).
	 * 
	 * @param accept		The format for which data from the service call should be returned. Typically "application/xml" or "application/json."
	 * @param contentType	The format for which POST body parameters will be sent from the client. Typically "application/xml" or "application/json."
	 * @param dateStr		Date in "D, d M Y H:i:s T" format.
	 * @param postBody		A string containing the JSON/XML POST request body parameter to be sent from the client.
	 * @param url			Full URL of the service you wish to call (EX. https://fakeApiURL.com/services/key?param1=test1&param2=test2,test3).
	 * 
	 * @return A String containing the message returned by the call. Null if the call fails.
	 * */
	public String sendTestAPICallPOST(String accept, String contentType, String dateStr, String postBody, String url) {
		try {
			URL parsedUrl = new URL(url);
			HttpURLConnection con = (HttpURLConnection) parsedUrl.openConnection(); 
			String md5 = "";
			String contentLength = "0";
			
			//First, if you have a post body you will be sending with the request, create an MD5 
			//hash of this post body and find the content length.
			if(postBody != null){
				MessageDigest digester = MessageDigest.getInstance("MD5");
				md5 = digester.digest(postBody.getBytes("UTF-8")).toString();
				contentLength = Integer.toString(postBody.length());
			}

			//Create the authorization property string for the request header.
	        //Format: MPA {key_id}:{signature}
			String signatureStr = getAuthenticationSignature(dateStr, parsedUrl.getPath(), contentType, "POST", md5);
			String authStr = "MPA " + this.keyId + ":" + signatureStr;
			
			//Set Connection Method to POST
			con.setRequestMethod("POST");
			
			//Set DoOutput to true so the post body can be sent 
			con.setDoOutput(true);
			
			//add request header
			con.setRequestProperty("Accept", accept);
			con.setRequestProperty("Authorization", authStr);
			con.setRequestProperty("Date", dateStr);
			con.setRequestProperty("Content-Length", contentLength);
			con.setRequestProperty("Content-MD5", md5);
			con.setRequestProperty("Content-Type", contentType);
			
			System.out.println("\n****Header Properties****");
			System.out.println("Accept : " + authStr);
			System.out.println("Authorization : " + authStr);
			System.out.println("Date : " + dateStr);
			System.out.println("Content-Length : " + contentLength);
			System.out.println("Content-MD5 : " + md5);
			System.out.println("Content-Type : " + contentType);
			System.out.println("*************************\n");
			
			//Send Post Body
			OutputStream os = con.getOutputStream();
			OutputStreamWriter osw = new OutputStreamWriter(os, "UTF-8");    
			osw.write(postBody);
			osw.flush();
			osw.close();
			os.close();
			con.connect();
			
			BufferedReader in = new BufferedReader(
			        new InputStreamReader(con.getInputStream()));
			String inputLine;
			StringBuffer response = new StringBuffer();

			while ((inputLine = in.readLine()) != null) {
				response.append(inputLine);
			}
			in.close();

			return response.toString();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
}
